﻿namespace UltraGridHideRowsAndColumnsAsExcel
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Linq;
    using System.Windows.Forms;
    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        private List<int> rowsIndexesToHide;
        private List<int> colsIndexesToHide;
        private int rows;
        private int cols;
        private int minVisibleSize;
        public Form1()
        {
            this.InitializeComponent();
            this.rowsIndexesToHide = new List<int>() { 0, 1, 2, 3, 7, 8, 10 };
            this.colsIndexesToHide = new List<int>() { 0, 1, 2, 3, 7, 8, 10 };
            this.rows = 15;
            this.cols = 15;
            this.minVisibleSize = 3;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.DataSource = this.GetData(this.rows, this.cols);
            this.ultraGrid1.AfterColPosChanged += UltraGrid1_AfterColPosChanged;
            this.ultraGrid1.AfterRowResize += UltraGrid1_AfterRowResize;
        }

        private void UltraGrid1_AfterRowResize(object sender, RowEventArgs e)
        {
            if (this.rowsIndexesToHide.Contains(e.Row.Index))
            {
                this.rowsIndexesToHide.Remove(e.Row.Index);
            }

            if (e.Row.GetSibling(SiblingRow.Next).Hidden == true)
            {
                e.Row.GetSibling(SiblingRow.Next).Hidden = false;
                this.ultraGrid1.EventManager.AllEventsEnabled = false;
                e.Row.GetSibling(SiblingRow.Next).Height = this.minVisibleSize;
                this.ultraGrid1.EventManager.AllEventsEnabled = true;
            }
        }

        private void UltraGrid1_AfterColPosChanged(object sender, AfterColPosChangedEventArgs e)
        {

            if (e.PosChanged == PosChanged.Sized)
            {
                UltraGridColumn col = e.ColumnHeaders.FirstOrDefault().Column;
                if (this.colsIndexesToHide.Contains(col.Index))
                {
                    this.colsIndexesToHide.Remove(col.Index);
                }

                if (col.Index < col.Band.Columns.Count-1 && col.Band.Columns[col.Index+1].Hidden == true)
                {

                    col.Band.Columns[col.Index + 1].Hidden = false;
                    this.ultraGrid1.EventManager.AllEventsEnabled = false;
                    col.Band.Columns[col.Index + 1].MinWidth = this.minVisibleSize;
                    col.Band.Columns[col.Index + 1].Width = this.minVisibleSize;
                    this.ultraGrid1.EventManager.AllEventsEnabled = true;
                } 
            }
        }

        private object GetData(int rows, int cols)
        {
            DataTable dt = new DataTable();

            for (int col = 0; col < cols; col++)
            {
                dt.Columns.Add(string.Format("Column #{0}", col + 1));
            }

            for (int row = 0; row < rows; row++)
            {
                DataRow addRow = dt.NewRow();
                foreach (DataColumn column in dt.Columns)
                {
                    addRow[column] = string.Format("{0}.{1}", row + 1, column.Ordinal + 1);
                }

                dt.Rows.Add(addRow);
            }

            return dt;
        }

        private void ultraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.Override.RowSelectorHeaderStyle = RowSelectorHeaderStyle.SeparateElement;

            e.Layout.Override.RowSizing = RowSizing.Free;
            e.Layout.Override.MinRowHeight = this.minVisibleSize;

            this.MakeColumnHidden(e.Layout.Bands[0].Columns);
        }

        private void MakeColumnHidden(ColumnsCollection cols)
        {

            foreach (UltraGridColumn col in cols)
            {
                int curentColumnIndex = col.Index;
                if (this.colsIndexesToHide.Contains(curentColumnIndex))
                {
                    if (curentColumnIndex > 0 &&
                        (col.Band.Columns[curentColumnIndex - 1].Width == this.minVisibleSize ||
                        col.Band.Columns[curentColumnIndex - 1].Hidden == true))
                    {
                        col.Hidden = true;
                    }
                    else
                    {
                        col.MinWidth = this.minVisibleSize;
                        this.ultraGrid1.EventManager.AllEventsEnabled = false;
                        col.Width = this.minVisibleSize;
                        this.ultraGrid1.EventManager.AllEventsEnabled = true;
                    }
                }
            }
        }

        private void ultraGrid1_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            this.MakeRowHidden(e.Row);
        }

        private void MakeRowHidden(UltraGridRow row)
        {
            
            int curentRowIndex = row.Index;
            if (this.rowsIndexesToHide.Contains(curentRowIndex))
            {
                if (curentRowIndex > 0 &&
                    (row.GetSibling(SiblingRow.Previous).Height == this.minVisibleSize ||
                    row.GetSibling(SiblingRow.Previous).Hidden == true))
                {
                    row.Hidden = true;
                }
                else
                {
                    this.ultraGrid1.EventManager.AllEventsEnabled = false;
                    row.Height = this.minVisibleSize;
                    this.ultraGrid1.EventManager.AllEventsEnabled = true;
                }
            }
        }
    }
}
